/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.FileField;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.components.PictureField;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.Base64;
import net.sf.ext.util.Resources;
import net.sf.ext.util.SwingUtil;
import net.sf.ext.util.Utilities;

/**
 * This view allows pictures to be imported and existing pictures
 * (pictures in the database) to be shown in its orginal size.
 * Currently the pictures are shown in a scaled way up to a maximum
 * resolution of 640x480. The user can scale it further by hand.
 * This number was chosen with a purpose; 640x480 ensures the picture
 * always being displayed in a resolution smaller then the monitor
 * resolution. 
 * 
 * Only pictures up to 100 kilobytes can be imported. Saving and loading
 * images can be very stressfull for the system. 100 kilobytes is a
 * randomly picked size..
 *   
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class PictureView extends JDialog {
    
    private byte[] bytes;
    private FileField ffOpenPicture;
    private PictureField picField;
    
    private JPanel panelPicture = new JPanel(false);
    
    // if the action performed was 'closed' set this to true.
    // clase is same as cancel, so this means the picture field
    // should not update itself
    private boolean closed = false;
    
    /**
     * Initializes the view and shows it. The view is always on top (modal).
     * @param url image to be displayed (can be null)
     */
    public PictureView(byte[] bytes) {
        super(SwingUtil.getRootFrame());
        
        this.bytes = bytes;
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });
        
        buildPanel();
        correctSize();

        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
        
        setModal(true); 
        setVisible(true);
    }
    
    public void close() {
        picField.flush();
        ffOpenPicture = null;
        picField = null;
        bytes = null;
        dispose();
    }
    
    public Object getValue() {
        Object value = bytes != null ? new String(Base64.encode(bytes)) : null;
        return value; 
    }
    
    /**
     * Indicates if the view was closed (cancel action). 
     */
    public boolean wasClosed() {
    	return closed;
    }
    
    /**
     * Corrects the size of picture:
     * Pictures bigger then 640x480 will be shown as 640x480
     */
    private void correctSize() {
        Dimension dim = picField.getPreferredSize();
        
        if (picField.isEmpty()) {
            panelPicture.setPreferredSize(new Dimension(dim.width, 
                    dim.width + 100));
            pack();
        } else if (dim.height > 480 && dim.width > 640) {
            panelPicture.setPreferredSize(new Dimension(640, 480));            
            pack();
        } else {
            panelPicture.setPreferredSize(new Dimension(dim.width + 20, dim.height + 20));
            pack();
        }
    }    
    
    private void savePicture() {
        try {
            File file = ffOpenPicture.getFile();
            
            if (file != null && file.length() > 512000) {
                new MessageBox(Resources.getText(DCMessages.msgImageFileIsTooLarge, "500"), MessageBox._WARNING);
                file = null;
                return;
            } else if (file != null) {
                bytes = Utilities.readFile(file);
                picField.setValue(new String(Base64.encode(bytes)));
                correctSize();

                boolean delete = DcSettings.getValueAsBoolean(Repository.settings.stDeleteImageFileAfterImport);
                if (delete) file.delete();
            } else { // file = null
                bytes = null;
                picField.setValue(null);
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);          
    }    
    
    /** 
     * Creates this panel..
     */
    private void buildPanel() {
        Color color = DcSettings.getValueAsColor(Repository.settings.stColor);
        
        panelPicture.setBackground(color);
        JPanel panelFile = new JPanel(false);
        panelFile.setBackground(color);
        JPanel panelActions = new JPanel(false);
        panelActions.setBackground(color);
        this.getContentPane().setBackground(color);

        // ************************************************************************
        // PICTURE PANEL
        // Set layout style
        panelPicture.setLayout(Layout.getGBL());
        panelPicture.setMaximumSize(new Dimension(640, 480));

        // Initialize components
        picField = new PictureField(true, false, false);
        String base64 = bytes != null ? new String(Base64.encode(bytes)) : null;
        picField.setValue(base64);
        
        // Build the panel
        panelPicture.add(picField,  Layout.getGBC( 0, 0, 1, 1, 40.0, 40.0
                                   ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                                    new Insets( 5, 5, 5, 5), 0, 0));

        //**********************************************************
        //File Action panel
        //**********************************************************
        panelFile.setLayout(Layout.getGBL());
        ffOpenPicture = new FileField(false, false, new JpegFileFilter());
        ffOpenPicture.addListener(new ActionImportImage());
        
        panelFile.add(ffOpenPicture,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                      new Insets( 5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Action panel
        //**********************************************************
        panelActions.setLayout(Layout.getGBL());
        
        JButton buttonSave = ComponentFactory.getButton(Resources.getText(DCLabels.lblSavePicture));
        JButton buttonDelete = ComponentFactory.getButton(Resources.getText(DCLabels.lblDeletePicture));
        JButton buttonClose = ComponentFactory.getButton(Resources.getText(DCLabels.lblClose));
        
        buttonSave.addActionListener(new ActionSave());
        buttonClose.addActionListener(new ActionClose());
        buttonDelete.addActionListener(new ActionDelete());
        
        panelActions.add(buttonSave,    Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                       ,GridBagConstraints.NORTH, GridBagConstraints.NONE,
                                        new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonDelete,  Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                                       ,GridBagConstraints.NORTH, GridBagConstraints.NONE,
                                        new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonClose,   Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                                       ,GridBagConstraints.NORTH, GridBagConstraints.NONE,
                                        new Insets( 5, 5, 5, 5), 0, 0));
        
        //**********************************************************
        //Main panel
        //**********************************************************
        getContentPane().setLayout(Layout.getGBL());
        setTitle(Resources.getText(DCLabels.lblPictureViewer));
        
        getContentPane().add(   panelPicture,  Layout.getGBC( 0, 0, 1, 1, 40.0, 40.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                               new Insets( 0, 0, 0, 0), 0, 0));
        getContentPane().add(   panelFile,     Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                               new Insets( 0, 0, 0, 0), 0, 0));
        getContentPane().add(   panelActions,  Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                               new Insets( 0, 0, 0, 0), 0, 0));
    }    
    
    private class ActionDelete implements ActionListener {
       public void actionPerformed(ActionEvent arg0) {
           bytes = null;
           picField.setValue(null);
           picField.updateUI();
           correctSize();
           
           Dimension location = Utilities.getCenteredWindowLocation(getSize());
           setLocation(location.width, location.height);           
       }
    }

    private class ActionClose implements ActionListener {
       public void actionPerformed(ActionEvent arg0) {
       	   closed = true;
           dispose();
       }
    }

    private class ActionSave implements ActionListener {
       public void actionPerformed(ActionEvent arg0) {
           dispose();
       }
    }
     
    /**
     * Imports the image. Images larger then 100 kilobytes are not allowed.
     */
    private class ActionImportImage implements ActionListener {
       public void actionPerformed(ActionEvent arg0) {
            savePicture();
       }
    }
    
    /**
     * Image file filter
     */
    private class JpegFileFilter extends FileFilter {
        public boolean accept(File file) {
            if (file.isDirectory()) {
                return true;
            } else if (Utilities.getExtension(file).equals("jpg") || 
                       Utilities.getExtension(file).equals("jpeg")|| 
                       Utilities.getExtension(file).equals("png") ||
                       Utilities.getExtension(file).equals("gif") ||
                       Utilities.getExtension(file).equals("bmp")) {
                return true;
            } else {
                return false;
            }
        }
        
        public String getDescription() {
            return Resources.getText(DCLabels.lblPicFileFilter);
        }
    }
}
