/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collection;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.SwingUtilities;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCTable;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.components.renderers.DCTableCellRenderer;
import net.sf.dc.console.movie.ImdbMovieSearchForm;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.processes.DataTask;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.resources.DCTooltips;
import net.sf.ext.util.Resources;

/**
 * Base panel for all panels which contain a JTable
 * 
 * This class facilities most of the functions needed to fill / clear /
 * update tables. It also provides a popup menu for the table and 
 * action buttons.
 *
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 1.9
 */ 
public abstract class TablePanel extends DCPanel {

    protected DCTable table;
    
    protected TableActionsPanel panelTableActions;
    protected PopupMenu popupMenu;

    protected final TableMouseListener tableMouseListener = new TableMouseListener();
    protected DataTask task;
    
    private final DCModule module;
    private final DCModule moduleChild;

    /**
     * Initializes this panel
     * @param module the module for which this panel is created
     * @param title the title of this panel
     * @param icon the icon of this panel
     */
    public TablePanel(DCModule module, String title, ImageIcon icon) {
        super(title, icon);  
        this.module = module;
    	this.moduleChild = module != null ? module.getChildModule() : null;
    }
    
    /**
     * Returns the child module (or null if it does not exist)
     */
    protected DCModule getChildModule() {
        return moduleChild;
    }
    
    /**
     * Indicates whether thids panel has a child panel
     */
    protected boolean hasChild() {
        return getChildModule() != null;
    }
    
    /**
     * Retrieves the module to which this panel belongs 
     */
    protected DCModule getModule() {
    	return module;
    }
    
    /**
     * Indicates whether there is a data task running at this moment
     */
    protected boolean isTaskRunning() {
        return task != null && task.isRunning();
    }  
    
    public void showFieldSelector() {
        table.openFieldSelector();
    }
    
    /**
     * Fills the table with a list of data crow objects
     * @param data list of data crow objects
     */
    public void fillTable(Collection objects) {
        table.fillTable(objects);
        String sItems = "" + table.getRowCount();
        updateProgressBar(0);
    	setStatus(Resources.getText(DCMessages.msgSearchResult, sItems));
    }

    /**
     * Mark fields as being required
     */
    public void setRequiredFields() {
    	table.setRequiredFields();
    }
    
    /**
     * Removes hidden fields from the search table.
     */
    public void removeHiddenFields(int[] indices) {
    	table.removeHiddenFields(indices);
    }
    
    /**
     * Updates the row with the given ID with the data (values)
     * @param ID the ID (Product Object ID)
     * @param values the new values
     */
    public void updateRow(String id, DCObject dco, boolean overwrite, boolean allowDeletes) {
        int row = table.getRowNumberWithID(id);
        table.updateRowAt(row, dco, overwrite, allowDeletes);
    }
    
    /**
     * Disables the save and clear buttons. 
     * Denies the popupmenu to appear
     */
    public void denyActions() {
    	table.removeMouseListener(tableMouseListener);    
    	panelTableActions.buttonClear.setEnabled(false);
        panelTableActions.buttonSave.setEnabled(false);
    }

    /**
     * (Re-) enables the save and clear buttons.
     * Alles the popupmenu to (re-) appear
     */
    public void allowActions() {
        table.addMouseListener(tableMouseListener);
        panelTableActions.buttonClear.setEnabled(true);
        panelTableActions.buttonSave.setEnabled(true);
    }
    
    
    
    /**
     * Opens the item form for the selected item
     */
    public void openItemForm(DCObject dco, boolean update) {
        dco.setRow(-1);
        int selectedRow = table.getSelectedRow();
        ItemForm itemForm = new ItemForm(    DataCrow.mainFrame,
                                             false,
                                             update,
                                             dco,
                                             getModule().isImdbUpdateAllowed(),
                                             getModule().isAmazonUpdateAllowed(),
                                             getClass(),
                                             selectedRow);
        itemForm.setVisible(true);
    }
    
    public void removeSelectedRows() {
        table.removeRows(table.getSelectedRows());
    }

    /**
     * Remove the rows from the table with the given indices
     */
    public void removeRows(String[] ids) {
        for (int i = 0; i < ids.length; i++) {
            int row = table.getRowNumberWithID(ids[i]);
            table.removeRow(row);
        }
    }
    
    /**
     * Cancels the currently running data task
     */
    public void cancelTask() {
        if (isTaskRunning()) {
            task.cancel();
            allowActions();
        }
        updateProgressBar(0);
    }
    
    /**
     * Returns all rows in the UI table as Data Crow objects.
     * Use carefully! In case of thousands of objects a huge amount
     * of memory is used. Make sure that getting the cached items only
     * isn't enough!
     */
    public DCObject[] getObjects() {
    	return table.getDCObjects();
    }
    
    /**
     * Clears this table of all data
     */
    public void clearTable() {
        updateProgressBar(0);
        table.clearTable();
        setStatus(Resources.getText(DCMessages.msgTableCleared));
    }
    
    /**
     * Tries to apply the current editing of the table cell.
     * If the current value cannot be applied, the editing is simply
     * cancelled (changes are discarded) 
     */
    public void cancelEdit() {
        int selectedRow = table.getSelectedRow();
        if (selectedRow != -1) {
            for (int i = 0; i < table.getColumnCount(); i++) {
                boolean success = false;
                try {
                    table.getCellEditor(selectedRow, i).stopCellEditing();
                    success = true;
                } catch (Exception ignore) {}
                
                if (!success) {
                    table.getCellEditor(selectedRow, i).cancelCellEditing();
                }
            }
        }
    }
    
    /**
     * Starts the Amazon internet update. The job is run in a seperate thread
     */
    public void startAmazonUpdate() {
        cancelEdit();
        
        int row = table.getSelectedRow();
        
        DCObject dco = null;
        if (row != -1) {
        	dco = table.getDCObject(row);
        } 
        
        AmazonSearchForm searchForm = new AmazonSearchForm(dco, DataCrow.mainFrame.getSelectedTab());
        searchForm.setVisible(true);
        
        if (dco != null) {
            searchForm.startSearch();
        }
    }

    
    /**
     * Starts the IMDB internet update. The job is run in a seperate thread
     */
    public void startIMDBUpdate() {
        cancelEdit();

        int row = table.getSelectedRow();
        DCObject dco = null;
        if (row != -1) {
            dco = table.getDCObject(row);
        } 
    	ImdbMovieSearchForm searchForm = new ImdbMovieSearchForm(dco, DataCrow.mainFrame.getSelectedTab(), row);
        searchForm.setVisible(true);
        
        if (dco != null) {
            searchForm.startSearch();
        }
    }
    
    public abstract void openItemForm();
    public abstract void createPdfReport();
    public abstract void addObjects(Collection objects);
    
    public void addObject(DCObject dco) {
        if (dco.getID() == null || dco.getID().equals("")) {
        	dco.setIDs();
        }
        table.addObject(dco);
    }

    /**
     * Menu shown when the user performs a right click on a table.
     * The menu is build up from information from the owner module.
     *  
     * @author rjwaal
     * @since 1.3
     * @version 1.00
     */
    public class PopupMenu extends JPopupMenu {
        JMenuItem menuOpen;
        JMenuItem menuSave;
        JMenuItem menuDelete;
        JMenuItem menuUpdateAll;
        JMenuItem menuIMDB;
        JMenuItem menuAmazon;
        JMenuItem menuRemoveRow;
        JMenuItem menuAddRow;
        
        JMenuItem menuReportPdf;
        
        JMenuItem menuFieldSelector;

        public PopupMenu(boolean showImdb,
                         boolean showAmazon,
                         boolean showDelete,
                         boolean showAddRemove,
                         boolean showOpen,
                         boolean showExport) {

            DCModule module = getModule();
            String name  = module.getObjectName();
            
            menuOpen = new JMenuItem(Resources.getText(DCLabels.lblOpenItem, ""),
                                     IconLibrary._icoMenuOpen16);
            menuSave = new JMenuItem(Resources.getText(DCLabels.lblSave, name),
                                     IconLibrary._icoMenuSave16);
            menuDelete = new JMenuItem(Resources.getText(DCLabels.lblDelete, name),
                                     IconLibrary._icoMenuRemove16);
            menuIMDB = new JMenuItem(Resources.getText(DCLabels.lblIMDBUpdate));
            menuAmazon = new JMenuItem(Resources.getText(DCLabels.lblAmazonUpdate));
            menuRemoveRow = new JMenuItem(Resources.getText(DCLabels.lblRemoveRow, name),
                                     IconLibrary._icoMenuRemove16);
            menuAddRow = new JMenuItem(Resources.getText(DCLabels.lblAddRow, name),
                                     IconLibrary._icoMenuAdd16);
            menuReportPdf = new JMenuItem(Resources.getText(DCLabels.lblCreatePdfReport));
            menuFieldSelector = new JMenuItem(Resources.getText(DCLabels.lblFieldSelector));
            menuUpdateAll = new JMenuItem(Resources.getText(DCLabels.lblUpdateAll));

            menuFieldSelector.addActionListener(new FieldSelectorAction());
            menuOpen.addActionListener(new OpenItemFormAction());
            menuRemoveRow.addActionListener(new RemoveRowAction());
            menuAddRow.addActionListener(new AddRowAction());
            
            menuReportPdf.addActionListener(new CreatePdfReport());
            
            menuUpdateAll.addActionListener(new UpdateAllAction());

            this.add(menuFieldSelector);
            this.add(menuUpdateAll);
            this.add(new JSeparator(JSeparator.HORIZONTAL));

            if (showOpen) {
                this.add(menuOpen);
            }
            
            this.add(menuSave);
            if (showDelete) {
                this.add(menuDelete);
            }
            if (showAddRemove) {
                this.add(menuRemoveRow);
                this.add(menuAddRow);
            }
            
            if (showImdb) {
                this.add(new JSeparator(JSeparator.HORIZONTAL));
                this.add(menuIMDB);
            }

            if (showAmazon) {
                this.add(new JSeparator(JSeparator.HORIZONTAL));
                this.add(menuAmazon);
            }
            
            if (showExport) {
                this.add(new JSeparator(JSeparator.HORIZONTAL));
                this.add(menuReportPdf);
            }
        }

        public void addAmazonActionListener(ActionListener listener) {
            menuAmazon.addActionListener(listener);
        }

        public void addDeleteActionListener(ActionListener listener) {
            menuDelete.addActionListener(listener);
        }

        public void addIMDBActionListener(ActionListener listener) {
            menuIMDB.addActionListener(listener);
        }

        public void addSaveActionListener(ActionListener listener) {
            menuSave.addActionListener(listener);
        }
    }

    /**
     * Panel which holds all buttons and their actions for tables.
     *  
     * @author rjwaal
     * @since 1.3
     * @version 1.00
     */    
    public class TableActionsPanel extends JPanel {
        private JButton buttonClear;
        private JButton buttonSave;
        private JButton buttonAddRow;
        private JButton buttonRemoveRow;
        private JButton buttonCancel;

        public TableActionsPanel(boolean showAddRemoveRow, JComponent[] extraComponents) {
            setLayout(Layout.getGBL());
            
            DCModule module = getModule();
            String name = module.getObjectName();

            // Create the components
            buttonSave = ComponentFactory.getButton(Resources.getText(DCLabels.lblSaveChanges));
            buttonClear = ComponentFactory.getButton(Resources.getText(DCLabels.lblClear));
            buttonAddRow = ComponentFactory.getButton(Resources.getText(DCLabels.lblAddRow, name));
            buttonRemoveRow = ComponentFactory.getButton(Resources.getText(DCLabels.lblRemoveRow, name));
            buttonCancel = ComponentFactory.getButton(Resources.getText(DCLabels.lblCancel));

            buttonClear.setMnemonic(KeyEvent.VK_C);
            buttonCancel.setMnemonic(KeyEvent.VK_X);

            buttonSave.setToolTipText(Resources.getText(DCTooltips.tpSaveChanges));
            buttonClear.setToolTipText(Resources.getText(DCTooltips.tpClear));
            buttonAddRow.setToolTipText(Resources.getText(DCTooltips.tpAddRow));
            buttonRemoveRow.setToolTipText(Resources.getText(DCTooltips.tpRemoveRow));
            buttonCancel.setToolTipText(Resources.getText(DCTooltips.tpCancel));

            buttonClear.addActionListener(new ClearButtonAction());
            buttonAddRow.addActionListener(new AddRowAction());
            buttonRemoveRow.addActionListener(new RemoveRowAction());
            buttonCancel.addActionListener(new CancelAction());

            // Build the panel
            if (showAddRemoveRow) {
                add(buttonAddRow,   Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));
                add(buttonRemoveRow,Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));
            }
            
            int counter = 2;
            if (extraComponents != null) {
                for (int i = 0; i < extraComponents.length; i++) {
                    add(extraComponents[i],  
                             Layout.getGBC(counter, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));
                    counter++;
                }
            }            

            add(buttonSave,     Layout.getGBC( counter++, 0, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                new Insets(5, 5, 5, 5), 0, 0));

            add(buttonClear,    Layout.getGBC( counter++, 0, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                new Insets(5, 5, 5, 5), 0, 0));
            add(buttonCancel,   Layout.getGBC( counter++, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                 new Insets(5, 5, 5, 5), 0, 0));
        }

        /** 
         * Allows additional action listener to be added to the 
         * save button. Sequence of execution of these actions cannot
         * be garuanteed.
         * @param listener action listener to be added
         */
        public void addSaveActionListener(ActionListener listener) {
            buttonSave.addActionListener(listener);
        }

        /** 
         * Removes action listeners from the save button. 
         * @param listener action listener to be removed 
         */        
        public void removeSaveActionListener(ActionListener listener) {
            buttonSave.removeActionListener(listener);
        }
    }

    /**
     * Opens an item
     */
    public class OpenItemFormAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            openItemForm();
        }
    }

    /**
     * Clears the table
     */
    public class ClearButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clearTable();
        }
    }

    /**
     * Opens the hidden / visible fields selector dialog 
     */
    public class FieldSelectorAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showFieldSelector();
        }        
    } 

    /**
     * Opens the dialog for updating all the values of one (or more)
     * columns in the UI table
     */
    public class UpdateAllAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
        	table.openUpdateAllDialog();
        }        
    }    

    /**
     * Adds an empty row to the table
     */
    public class AddRowAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            addObject(getModule().getDCObject());
        }
    }

    /**
     * Removes a row (or multiple rows) from the table
     */
    public class RemoveRowAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            removeSelectedRows();
        }
    }

    /**
     * Removes a row (or multiple rows) from the table
     */
    public class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancelTask();
        }
    }
    
    /**
     * Listens on right clicks:
     * shows the popup menu 
     * 
     * Listens on double clicks:
     * opens the item form for the row on which is double clicked
     */
    private class TableMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (SwingUtilities.isRightMouseButton(e)) {
                if (table.getSelectedRows() != null &&
                    table.getSelectedRows().length < 2) {
                    int row = table.rowAtPoint(e.getPoint());
                    int col = table.getSelectedColumn() != -1 ? table.getSelectedColumn() :
                                                                table.columnAtPoint(e.getPoint());
                    table.addRowSelectionInterval(row, row);
                    table.addColumnSelectionInterval(col, col);
                    
                    ((DCTableCellRenderer) table.getCellRenderer(row, col)).
                        getTableCellRendererComponent(
                                table, 
                                table.getValueAt(row, col, true),
                                true,
                                true,
                                row,
                                col);
                    
                }
                popupMenu.show(table, e.getX(), e.getY());
            }

            if (e.getClickCount() == 2) {
            	table.ignoreEdit(true);
            	cancelEdit();
            	table.ignoreEdit(false);
                openItemForm();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }
    
    /**
     * Opens the PDF creation dialog 
     */
    public class CreatePdfReport implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            createPdfReport();
        }        
    }
}
