/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;

import javax.swing.CellRendererPane;
import javax.swing.JComponent;
import javax.swing.JTextPane;
import javax.swing.JToolTip;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicToolTipUI;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import net.sf.dc.core.Repository;
import net.sf.dc.messages.Messages;
import net.sf.dc.settings.DcSettings;

/**
 * UI implementation of a multiple line tooltip
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 1.00
 */
public class MultiLineToolTipUI extends BasicToolTipUI {

    private final static MultiLineToolTipUI sharedInstance = new MultiLineToolTipUI();
    private CellRendererPane rendererPane;
    private static JTextPane textArea;
    private boolean containsText = true;

    public MultiLineToolTipUI() {
        super();
        if (textArea == null) {
            textArea = ComponentFactory.getTextPane();
            textArea.setDocument(new HTMLDocument());
            textArea.setEditorKit(new HTMLEditorKit());
            textArea.setBackground(new Color(239,227,178));
        }
    }

    public static ComponentUI createUI(JComponent c) {
        return sharedInstance;
    }
    
    public void installUI(JComponent c) {
        super.installUI(c);
        rendererPane = new CellRendererPane();
        rendererPane.add(textArea);
        c.add(rendererPane);
    }
    
    public void uninstallUI(JComponent c) {
        super.uninstallUI(c);
        c.remove(rendererPane);
        rendererPane = null;
    }
    
    public void paint(Graphics g, JComponent c) {
        if (containsText) {
            Dimension size = c.getSize();
            rendererPane.paintComponent(g, textArea, c, 1, 1, size.width - 1, size.height - 1, true);
        } else {
            g.dispose();
        }
    }
    
    private void setHtmlText(String text) {
        String s = getHtmlText(text);
        
        HTMLDocument document = (HTMLDocument) textArea.getDocument();
        HTMLEditorKit kit = (HTMLEditorKit) textArea.getEditorKit();
        
        try {
            textArea.setText("");
            kit.insertHTML(document, 0, s, 0, 0, null);
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }
    
    private String getHtmlText(String text) {
      String html = "";
      int length = text != null ? text.length() : 0;
      if (length > 1) { 
          text = length > 1000 ? text.substring(0, 1000) + "...." : text;
          StringBuffer sb = new StringBuffer();
          sb.append("<html><body><table ");
          sb.append(getHtmlStyle(length));
          sb.append("><tr><td>");
          sb.append(text);
          sb.append("</td></tr></table></body></html>");
          html = sb.toString();
          containsText = true;
      } else {
          containsText = false;
      }
      return html;
    }
    
    private String getHtmlStyle(int length) {
      Font font = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);

      StringBuffer sb = new StringBuffer();
      sb.append("style=");
      sb.append('"');
      sb.append("background:FFFFE9;");
      //sb.append("color:A80015;font-style:italic;");
      sb.append("font-family:");
      sb.append(font.getFamily());
      sb.append(";");
      sb.append("font-size:");
      sb.append(font.getSize());
      sb.append(";");            
    
      if (length > 100) {
          sb.append("width:");
          sb.append(400);
          sb.append(";");               
      }
      sb.append('"');            
      return sb.toString();
    }       
    
    public Dimension getPreferredSize(JComponent c) {
        String tipText = ((JToolTip)c).getTipText();
        Dimension dim = new Dimension(0,0);
        setHtmlText(tipText);
        
        if (containsText) {
            int width = ((MultiLineToolTip)c).getFixedWidth();
            int columns = ((MultiLineToolTip)c).getColumns();
            
            if (columns > 0) {
                textArea.setSize(0,0);
                textArea.setSize( textArea.getPreferredSize() );
            } else if (width > 0) {
                Dimension d = textArea.getPreferredSize();
                d.width = width;
                d.height++;
                textArea.setSize(d);
            } 
            dim = textArea.getPreferredSize();
            dim.height += 1;
            dim.width += 1;
        }
        return dim;
    }
    
    public Dimension getMinimumSize(JComponent c) {
        return getPreferredSize(c);
    }
    
    public Dimension getMaximumSize(JComponent c) {
        return getPreferredSize(c);
    }
}
