/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JComponent;

import net.sf.dc.console.PictureView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.resources.DCTooltips;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;
import net.sf.ext.util.Base64;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * Shows a scaled (or not scaled) picture. Also has a zoom function.
 * On a double click the picture view is opened.
 * 
 * @see net.sf.dc.console.PictureView
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 1.00
 */
public class PictureField extends JComponent implements ISettingsComponent {

    byte[] bytes = null;
    private Image img = null;
    
    private boolean scaled = true;
    private boolean thumbnail = false;
    private Dimension size = null;
    
    private Dimension iconSize = null;
    private Insets insets = new Insets(0, 0, 0, 0);

    /**
     * Creates this field
     * @param allowZoom Indicates if the picture view can be opened
     */
    public PictureField(boolean allowZoom) {
        this(true, allowZoom, false);
    }

    /**
     * Creates this field
     * @param scaled Indicates whether the picture is allowed to be scaled
     * @param allowZoom Indicates if the picture view can be opened
     * @param thumbnail Only show a small version of the image
     */
    public PictureField(boolean scaled, boolean allowZoom, boolean thumbnail) {
        this.scaled = scaled;
        this.thumbnail = thumbnail;
        
        if (allowZoom) {
            setToolTipText(Resources.getText(DCTooltips.tpPictureField));
            addMouseListener(new OpenPictureFrameAction());
        }        
    }
    
    /**
     * Sets a value on this field
     * @param o fully qualified path or an URL
     */
    public void setValue(Object o) {
        flushImage();
        
        try {
            if (o == null) {
                bytes = null;
            } else {
                if (o instanceof URL) {
                    URL url = (URL) o;
                    bytes = Utilities.readFile(new File(url.getFile()));
                } else {
                    String value = (String) o;
                    if (value.endsWith("jpg")) {
                        URL url;
                        if (!value.startsWith("file:")) 
                        	url = new URL("file:" + value);
                        else 
                            url = new URL(value);
    
                        bytes = Utilities.readFile(new File(url.getFile()));
                    } else if (value.length() > 1) {
                        bytes = Base64.decode(value.toCharArray());
                    }
                } 
            }
        } catch (Exception exp) {
            Messages.add(Resources.getText(DCMessages.msgCouldNotLoadPicture), Messages._ERROR);
            Messages.add(exp);
        }        
        
        if (bytes != null) {
            ImageIcon icon = new ImageIcon(bytes);
            int height = icon.getIconHeight();
            int width = icon.getIconWidth();
            iconSize = new Dimension(width,height);
            img = icon.getImage();
        }
        revalidate();
        repaint();
    }
    
    /**
     * Explicit call for flushing the image
     */
    public void flush() {
        flushImage();
    }

    /**
     * Flushes the images and sets all the references to null
     */
    private void flushImage() {
    	if (img != null) {
    		img.flush();
            img = null;
        }
    }
    
    public boolean isEmpty() {
    	return img == null;
    }
    
    /**
     * Returns the image as a BASE64 string
     */
    public Object getValue() {
        String value = bytes != null ? new String(Base64.encode(bytes)) : null;
        return value;
    }
    
    private Image getImage() {
    	return img;
    }
    
    /**
     * Indicates if scaling is allowed. Scaling is not allowed for very 
     * small pictures as otherwise they will just become disrupted..
     */
    private boolean scalingAllowed() {
        if (scaled && (iconSize.height >= 50 && iconSize.width >= 50)) {
        	return true;
        } else {
        	return false;
        }
    }

    
    /**
     * Paints the image. This is either done in a scaled way 
     * (@see #scalingAllowed(Dimension)) or in it is orginal state
     */
    public void paint(Graphics g) {
        insets = getInsets(insets);
        size = getSize(size);
        
        Image image = getImage();
        
        if (image != null && scalingAllowed()) {
            int imageWidth = image.getWidth(null);
            int imageHeight = image.getHeight(null);
            
            int w = 0;
            int h = 0;
            if (thumbnail) {
                w =  Math.min(size.width, image.getWidth(this)/ 5);
                h = Math.min(size.height, image.getHeight(this)/ 5);
            } else {
                w =  Math.min(size.width, image.getWidth(this));
                h = Math.min(size.height, image.getHeight(this));
                
                double scaledRatio = (double) w / (double) h;
                double imageRatio = (double) imageWidth / (double) imageHeight;
                if (scaledRatio < imageRatio) {
                    h = (int)(w / imageRatio);
                } else {
                    w = (int)(h * imageRatio);
                }                
            }
            
            int spaceleft = getInset(size.width, w);
            int spacetop = getInset(size.height, h);

            g.drawImage(image, spaceleft, spacetop, w, h, this);
        } else if (image != null){
            int imageWidth = image.getWidth(null);
            int imageHeight = image.getHeight(null);

            int spaceleft = getInset(size.width, imageWidth);
            int spacetop = getInset(size.height, imageHeight);

            g.drawImage(image, spaceleft, spacetop, this);
        } else {
            int w =  Math.min(size.width, IconLibrary._icoNoPic.getWidth(this));
            int h = Math.min(size.height, IconLibrary._icoNoPic.getHeight(this));
            int spaceleft = getInset(size.width, w);
            int spacetop = getInset(size.height, h);

            g.drawImage(IconLibrary._icoNoPic, spaceleft, spacetop, w, h, this);
        } 
        g.dispose();
    }     

    private int getInset(int sizeComp, int sizePic) {
        int remainingSpace = sizeComp - sizePic;
        if (remainingSpace >= 2) {
        	return remainingSpace / 2;
        } else {
        	return 0;
        }
    }
    
    /**
     * Calculates the minimum size for this component
     */
    public Dimension getMinimumSize() {
        int imgw = 32, imgh = 32;
        Image image = getImage();
        if (thumbnail && image != null) {
            imgw =  image.getWidth(this)/ 5;
            imgh = image.getHeight(this)/ 5;
        } else if (image != null) {
            imgw = image.getWidth(this);
            imgh = image.getHeight(this);
        }
        insets = getInsets(insets);
        return new Dimension(
            insets.left + Math.max(32, imgw / 10) + insets.right,
            insets.top + Math.max(32, imgh / 10) + insets.bottom);
    }

    /**
     * Calculates the preferred size for this component
     */
    public Dimension getPreferredSize() {
        int imgw = 32, imgh = 32;
        Image image = getImage();
        if (thumbnail && image != null) {
            imgw =  image.getWidth(this)/ 5;
            imgh = image.getHeight(this)/ 5;
        } else if (image != null) {
            imgw = image.getWidth(this);
            imgh = image.getHeight(this);
        }
        
        insets = getInsets(insets);
        return new Dimension(
            insets.left + imgw + insets.right,
            insets.top + imgh + insets.bottom);
    }
    


    /**
     * Calculates the maximum size for this component
     */
    public Dimension getMaximumSize() {
        return getPreferredSize();
    }

    /**
     * Opens the picture viewer (@see PictureView)
     */
    public void showFullPicture() {
        PictureView view = new PictureView(bytes);
        if (!view.wasClosed()) {
        	setValue(view.getValue());
        }
    }
    
    public class OpenPictureFrameAction implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                showFullPicture();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }  
}