/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Component;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.filechooser.FileFilter;

import net.sf.dc.core.Repository;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.SwingUtil;

/**
 * Creates and opens files using the JFileChooser class from Java.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class BrowserDialog extends JFrame {

    private final JFileChooser browser = new JFileChooser();

    /**
     * Initializes this dialog
     * @param sTitle title of this dialog as shown to the user
     */
    public BrowserDialog(String sTitle) {
        browser.setBackground(DcSettings.getValueAsColor(Repository.settings.stColor));
        browser.setMultiSelectionEnabled(false);
        this.setTitle(sTitle);
    }
    
    /**
     * Initializes this dialog and applies a file filter.
     * The file filter will restrict the files and directories 
     * shown.
     * @param sTitle title of this dialog as shown to the user
     * @param filter the file filter
     */
    public BrowserDialog(String sTitle, FileFilter filter) {
        this(sTitle);
        
        if (filter != null) {
            browser.setFileFilter(filter);
        }
    }

    /**
     * Opens a dialog to select a directory and returns it (or null)
     * @return The selected file (can be null)
     */
    public File showSelectDirectoryDialog(Component c, File file) {
        setCurrentDirectory(file);
        browser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        
        if (c != null) { 
        	browser.showOpenDialog(c);
        } else {
        	browser.showOpenDialog(SwingUtil.getRootFrame());
        }        
        
        File f = browser.getSelectedFile(); 
        rememberUsedDirectory(f);
        return f;
    }
    
    /**
     * Opens a save file dialog and returns the selected file or null
     * @return The selected file (can be null)
     */
    public File showCreateFileDialog(Component c, File file) {
        setCurrentDirectory(file);
        browser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        
        if (c != null) { 
        	browser.showSaveDialog(c);
        } else {
        	browser.showSaveDialog(SwingUtil.getRootFrame());
        }

        File f = browser.getSelectedFile(); 
        rememberUsedDirectory(f);
        return f;
    }
    
    /**
     * Returns the selected file (can be null)
     * @return The selected file (can be null)
     */
    public File showOpenFileDialog(Component c, File file) {
        setCurrentDirectory(file);
        browser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        
        if (c != null) { 
        	browser.showOpenDialog(c);
        } else {
        	browser.showOpenDialog(SwingUtil.getRootFrame());
        }

        File f = browser.getSelectedFile(); 
        rememberUsedDirectory(f);
        return f;
    }
    
    /**
     * Applies the current directory on the browser. The current 
     * directory can either be the specified file or the path as stored
     * in the settings (last used directory)
     * @param file current directory (is allowed to be null)
     */
    private void setCurrentDirectory(File file) {
        if (file != null) {
            browser.setCurrentDirectory(file);
        } else {
            String s = DcSettings.getValueAsString(Repository.settings.stLastDirectoryUsed);
            file = new File(s);
            browser.setCurrentDirectory(file);
        }
    }
    
    /**
     * Sets the setting 'Last Directory Used'. This enables this browser
     * to reuse the last selected location.
     * @param file the selected directory or file
     */
    private void rememberUsedDirectory(File file) {
        if (file != null) {
            DcSettings.setStringAsValue(Repository.settings.stLastDirectoryUsed,
                                      file.toString());
        }        
    }
}
