/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.DCMenuBar;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.QuickViewPane;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.movie.InsertMoviePanel;
import net.sf.dc.console.movie.MovieQuickViewPane;
import net.sf.dc.console.movie.SearchMoviePanel;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.objects.reporting.ReportFieldDefinitions;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportMovieCD;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;

/**
 * The Movie module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 1.00
 */
public class MovieModule extends DCModule {
    
    private final DCMenuBar menuBar;
    private final ImportCDDialog importCDDialog = 
        new ImportCDDialog(DataCrow.mainFrame, 
                           Resources.getText(DCLabels.lblMovieFileImport));
    
    /**
     * Initializes this module
     */
    public MovieModule() {
        super(DCModules._MOVIE,
              Resources.getText(DCLabels.lblModuleMovie),
              "movies",
              "mov",
              "movies",
              true,
              false,
              false);
        menuBar = new DCMenuBar(this);
    }

    public DCMenuBar getMenuBar() {
        return menuBar;
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoMovieModule;
    }
    
    public String getObjectName() {
        return Resources.getText(DCLabels.lblMovie);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(Resources.getText(DCLabels.lblTools));
        JMenuItem menuImport = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuImport16,
                                                Resources.getText(DCLabels.lblMovieFileImport));
        menuImport.addActionListener(new ImportCDAction());
        menu.add(menuImport);
        
        return menu;
    }
    
    /**
     * Return the object managed by this module 
     * @see Movie
     */
    public DCObject getDCObject() {
        return new Movie();
    }
    
    public QuickViewPane getQuickView() {
        return new MovieQuickViewPane();
    }
        
    /**
     * Create the insert and search panel
     */
    protected void initializeUI() {
        insertPanel = new InsertMoviePanel();
        searchPanel = new SearchMoviePanel();
    }
        
    public String getHiddenFieldsSettingsKey() {
        return Repository.settings.stHiddenFieldsMovie;
    }

    public String getRequiredFieldsSettingsKey() {
        return Repository.settings.stMovieRequiredFields;
    }
    
    public void startImport() {
        importCDDialog.setModule(DCModules.getModule(DCModules._MOVIE));
        importCDDialog.setVisible(true);
    }
    
    /**
     * Returns the movie CD import process
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmDirs indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */    
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
							     			  String nameOfCD) {

        return new ImportMovieCD(ui,
                                 sourceDir,
                                 confirmDirs,
                                 recursiveDirs,
                                 nameOfCD);
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(Movie._ID, false, 0);
        definitions.addDefinition(Movie._A_TITLE, true, 0);
        definitions.addDefinition(Movie._B_TITLE_LOCAL, false, 0);
        definitions.addDefinition(Movie._C_DESCRIPTION, true, 0);
        definitions.addDefinition(Movie._D_WEBPAGE, false, 0);
        definitions.addDefinition(Movie._E_GENRE, true, 0);
        definitions.addDefinition(Movie._F_ACTORS, false, 0);
        definitions.addDefinition(Movie._G_DIRECTOR, false, 0);
        definitions.addDefinition(Movie._H_COUNTRY, false, 0);
        definitions.addDefinition(Movie._I_YEAR, true, 0);
        definitions.addDefinition(Movie._J_PLAYLENGTH, true, 0);
        definitions.addDefinition(Movie._K_FILENAME, false, 0);
        definitions.addDefinition(Movie._L_VIDEOCODEC, false, 0);
        definitions.addDefinition(Movie._M_AUDIOCODEC, false, 0);
        definitions.addDefinition(Movie._N_WIDTH, false, 0);
        definitions.addDefinition(Movie._O_HEIGHT, false, 0);
        definitions.addDefinition(Movie._P_FPS, false, 0);
        definitions.addDefinition(Movie._Q_FRAMES, false, 0);
        definitions.addDefinition(Movie._R_AUDIOBITRATE, false, 0);
        definitions.addDefinition(Movie._S_AUDIOSAMPLINGRATE, false, 0);
        definitions.addDefinition(Movie._T_AUDIOCHANNEL, false, 0);
        definitions.addDefinition(Movie._U_VIDEOBITRATE , false, 0);
        definitions.addDefinition(Movie._V_PICTUREFRONT , true, 0);
        definitions.addDefinition(Movie._W_PICTUREBACK , false, 0);
        definitions.addDefinition(Movie._X_PICTURECD , false, 0);
        definitions.addDefinition(Movie._Y_LANGUAGE, false, 0);
        definitions.addDefinition(Movie._Z_AUDIOLANGUAGE , false, 0);
        definitions.addDefinition(Movie._1_CERTIFICATION , false, 0);
        definitions.addDefinition(Movie._2_FILESIZE , false, 0);
        definitions.addDefinition(Movie._3_LOCATION , false, 0);
        definitions.addDefinition(Movie._4_RATING , true, 0);
        return definitions;
    }      
    
    /**
     * Initalizes all the fields
     */
    protected void initializeFields() {
        fields = new DCFieldDefinition[32];
        int counter = 0; 
        fields[Movie._ID] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblID),
                                        true,
                                        false,
                                        false,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");
        fields[Movie._A_TITLE] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblTitle),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Title");
        fields[Movie._B_TITLE_LOCAL] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblTitleLocal),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Title2");
        fields[Movie._C_DESCRIPTION] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblDescription),
                                        false,
                                        true,
                                        false,
                                        8000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Description");
        fields[Movie._D_WEBPAGE] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblWebPage),
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._URLFIELD,
                                        Repository.datatypes._STRING,
                                        "Webpage");                                       
        fields[Movie._E_GENRE] = new DCFieldDefinition(counter++, 
                                        Resources.getText(DCLabels.lblGenre),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Genre");                                     
        fields[Movie._F_ACTORS] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblActors),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Actors");   
        fields[Movie._G_DIRECTOR] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblDirector),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Director");     
        fields[Movie._H_COUNTRY] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblCountry),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Country");    
        fields[Movie._I_YEAR] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblYear),
                                        false,
                                        true,
                                        false,
                                        4,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Year");     
        fields[Movie._J_PLAYLENGTH] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblPlayLength),
                                        false,
                                        false,
                                        false,
                                        10,
                                        ComponentFactory._TIMEFIELD,
                                        Repository.datatypes._INTEGER,
                                        "PlayLength"); 
        fields[Movie._K_FILENAME] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblFileName),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Filename");  
        fields[Movie._L_VIDEOCODEC] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblVideoCodec),
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "VideoCodec");   
        fields[Movie._M_AUDIOCODEC] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblAudioCodec),
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "AudioCodec");  
        fields[Movie._N_WIDTH] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblWidth),
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Width");                                         
        fields[Movie._O_HEIGHT] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblHeight),
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Height");                                         
        fields[Movie._P_FPS] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblFPS),
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Fps");                                         
        fields[Movie._Q_FRAMES] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblFrames),
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Frames");                                         
        fields[Movie._R_AUDIOBITRATE] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblAudioBitRate),
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "AudioBitRate");                                         
        fields[Movie._S_AUDIOSAMPLINGRATE] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblAudioSampleRate),
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "AudioSampleRate");                                         
        fields[Movie._T_AUDIOCHANNEL] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblAudioChannel),
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "AudioChannels");                                         
        fields[Movie._U_VIDEOBITRATE] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblVideoBitRate),
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "VideoBitRate");                                         
        fields[Movie._V_PICTUREFRONT] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblPictureFront),
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureFront");                                         
        fields[Movie._W_PICTUREBACK] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblPictureBack),
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureBack");                                         
        fields[Movie._X_PICTURECD] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblPictureCD),
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureCD");  
        fields[Movie._Y_LANGUAGE] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblLanguage),
                                        false,
                                        true,
                                        false,
                                        30,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Language");        
        fields[Movie._Z_AUDIOLANGUAGE] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblAudioLanguage),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "AudioLanguage");   
        fields[Movie._0_SUBTITLELANGUAGE] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblSubtitleLanguage),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "SubtitleLanguage");           
        fields[Movie._1_CERTIFICATION] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblCertification),
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Certification");
        fields[Movie._2_FILESIZE] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblFilesize),
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Filesize");              
        fields[Movie._3_LOCATION] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblLocation),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Location");       
        fields[Movie._4_RATING] = new DCFieldDefinition(counter++, Resources.getText(DCLabels.lblRating),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._RATINGCOMBOBOX,
                                        Repository.datatypes._INTEGER,
                                        "Rating");                
    }
    
    public DCObject[] getDefaultData() {
        return new DCObject[0];
    }
    
    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }      
}
