/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.db;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.util.Properties;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.Resources;

/**
 * Maintains and creates the connection to the database
 *
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class DatabaseConnection {

	public static boolean bConnected = false;
    public static boolean bServerConnection = false;
    
    /**
     * Creates the connection to the database. It shows the login dialog
     * over and over again until valid data is present to connect to the
     * database 
     */
    public static Connection connect() {
        Connection connection = null;
    	try {
            Class.forName(DcSettings.getValueAsString(Repository.settings.stDatabaseDriver));
            String sName = DcSettings.getValueAsString(Repository.settings.stConnectionString);
            
            String sConnectionString = Repository.dataBase.localString + "./data/" + sName;
            if (sName.startsWith("//")) {
                // connect to a database server
                sConnectionString = Repository.dataBase.serverString + sName;
                bServerConnection = true;
            } else if (sConnectionString.indexOf("hsql") != -1) {
                // establish connection to the local database
                bServerConnection = false;
            }
            
            connection = DriverManager.getConnection(sConnectionString, "SA", "");
            boolean needsConversion = checkAndCorrectSettings("./data/" + sName);
            
            connection.setAutoCommit(true);
            
            if (needsConversion) {
                new MessageBox(Resources.getText(DCMessages.msgConversionNeeded), 
                               MessageBox._INFORMATION);
                boolean success = new DatabaseConversion().start(connection);
                if (!success) {
                	new MessageBox(Resources.getText(DCMessages.msgConversionFailed), 
                                   MessageBox._ERROR);
                    System.exit(1);
                }
            }
            
    	} catch (Exception exp) {
            Messages.add(exp);      
            if (exp.getMessage().startsWith("socket creation error")) {
                new MessageBox(Resources.getText(DCMessages.msgServerCouldNotBeReached), MessageBox._ERROR);
            } else {
                new MessageBox(exp.getMessage(), MessageBox._ERROR);
            }
            System.exit(1);
    	}
        return connection;
    }
    
    /**
     * Modifies the settings of the HSQLDB database
     * @param database the database (file-)name
     */
    private static boolean checkAndCorrectSettings(String database) {
        boolean needsConversion = false;
    	try {
            File file = new File(database + ".properties");
            
            FileInputStream in = new FileInputStream(file);
            Properties properties = new Properties();
            properties.load(in);
            
            String s = properties.getProperty("hsqldb.script_format");
            if (s.equals("3")) {
                needsConversion = true;
            }
            
            properties.setProperty("hsqldb.cache_size_scale", "14");
            properties.setProperty("hsqldb.cache_scale", "14");
            properties.setProperty("runtime.gc_interval", "0");
            properties.setProperty("sql.compare_in_locale", "true");
            
            FileOutputStream out = new FileOutputStream(file);
            properties.store(out, "");
            out.close();
        } catch (Exception exp) {
        	Messages.add(Resources.getText(DCMessages.msgUnableToChangeDbSettings), 
                         Messages._ERROR);
        }
        return needsConversion;
    }
    
    /**
     * Upgrades the database. A conversion was needed for the database used
     * with Data Crow alpha and the later versions (LONGVARCHAR delivered 
     * some nasty errors). This method upgrades the database.
     * First a backup is created of the original database. 
     * 
     * @param databaseName name of the to be upgraded database
     */
//    private static boolean updateHSQLDB(String databaseName) {
//        try {
//        	File file = new File("./data/" + databaseName + ".script");
//            
//            if (!file.exists()) {
//                return true;
//            }
//            
//        	RandomAccessFile raf = new RandomAccessFile(file, "r");
//
//            long filePointer = 0;
//            long fileLength  = raf.length();
//            boolean upgrading = false;
//            
//            StringBuffer sb = new StringBuffer();
//            while (filePointer < fileLength) {
//                String line = raf.readLine();
//                if (line.startsWith("CREATE TABLE")) {
//                	sb.append(line);
//                    filePointer = raf.getFilePointer();
//                } else {
//                	break;
//                }
//            }
//            raf.close();
//
//            if (sb.indexOf("LONGVARCHAR") > -1) {
//                String sBackupFile = "./data/" + databaseName + ".script.bck";
//                if (!upgrading) {
//                	new QuestionBox(Resources.getText(DCMessages.msgUpgradeNeeded, 
//                                    sBackupFile), MessagesWriter.logForm);
//                    upgrading = true;
//                }
//                
//                if (QuestionBox.result.booleanValue()) {
//                    File backup = new File(sBackupFile);
//                    file.renameTo(backup);          
//                    
//                    file = new File("./data/" + databaseName + ".script");
//                    RandomAccessFile rafTarget = new RandomAccessFile(file, "rw");
//                    
//                    RandomAccessFile rafSource = new RandomAccessFile(backup, "r");
//                    
//                    filePointer = rafSource.getFilePointer();
//                    boolean headerDone = false;
//                    while (filePointer < fileLength) {
//                        String s = rafSource.readLine();
//                        if (!headerDone && s.startsWith("CREATE TABLE")) {
//                        	s = s.replaceAll("LONGVARCHAR", "VARCHAR");
//                        } else {
//                            headerDone = true;
//                        }
//                        
//                        if (s != null) {
//                            rafTarget.write(s.getBytes());
//                            rafTarget.write('\n');
//                        } 
//                        filePointer = rafSource.getFilePointer();
//                    }
//                    
//                    new MessageBox(Resources.getText(DCMessages.msgUpgradeSuccessFul), 
//                                   MessageBox._INFORMATION);
//                    rafSource.close();
//                    rafTarget.close();
//                } else {
//                    return false;   
//                }
//            } 
//        } catch (Exception exp) {
//            new MessageBox(Resources.getText(DCMessages.msgCouldNotUpgrade, '\n' + exp.getMessage()), 
//                           MessageBox._ERROR);
//            return false;
//        }
//        return true;
//    }
}
